@@ -98,6 +98,14 @@ class Freezer:
         # include-msvcr is used on Windows, but not in MingW
         self.include_msvcr: bool = IS_WINDOWS and bool(include_msvcr)
         self.target_dir = target_dir
+        self.default_bin_includes: list[str] = self._default_bin_includes()
+        self.default_bin_excludes: list[str] = self._default_bin_excludes()
+        self.default_bin_path_includes: list[str] = (
+            self._default_bin_path_includes()
+        )
+        self.default_bin_path_excludes: list[str] = (
+            self._default_bin_path_excludes()
+        )
         self.bin_includes: list[str] = self._validate_bin_file(bin_includes)
         self.bin_excludes: list[str] = self._validate_bin_file(bin_excludes)
         self.bin_path_includes: list[str] = self._validate_bin_path(
@@ -127,10 +135,9 @@ class Freezer:
             zip_filename = Path(zip_filename).with_suffix(".zip").name
             self.zip_filename = self.target_dir / "lib" / zip_filename
 
-        self._verify_configuration()
-
         self._symlinks: set[tuple[Path, Path, bool]] = set()
         self.files_copied: set[Path] = set()
+        self.modules_copied: list[Module] = []
         self.finder: ModuleFinder = self._get_module_finder()
 
     @property
@@ -190,9 +197,15 @@ class Freezer:
         if self.silent < 1:
             print(f"copying {source} -> {target}")
         shutil.copyfile(source, target)
-        shutil.copystat(source, target)
         if include_mode:
             shutil.copymode(source, target)
+            shutil.copystat(source, target)
+        else:
+            try:
+                shutil.copystat(source, target)
+            except OSError:
+                if self.silent < 3:
+                    print("WARNING: unable to copy file metadata:", target)
         self.files_copied.add(target)
 
         # handle post-copy tasks, including copying dependencies
@@ -216,9 +229,9 @@ class Freezer:
                 if source.is_dir():
                     source_subdir = source_dir / source_name
                     excludes_subdir = {
-                        ".".join(m.split(".")[1:])
+                        m.split(f"{source_name}.", 1)[1]
                         for m in excludes
-                        if m.split(".")[0] == source_name
+                        if m.startswith(f"{source_name}.")
                     }
                     copy_tree(source_subdir, target, excludes_subdir)
                 else:
@@ -230,9 +243,9 @@ class Freezer:
             print(f"copying data from package {module_name}...")
         # do not copy the subfolders which belong to excluded modules
         excludes = {
-            ".".join(m.split(".")[1:])
+            m.split(f"{module_name}.", 1)[1]
             for m in self.finder.excludes
-            if m.split(".")[0] == module_name
+            if m.startswith(f"{module_name}.")
         }
         copy_tree(source_dir, target_dir, excludes)
 
@@ -279,39 +292,15 @@ class Freezer:
         finder.include_file_as_module(
             get_resource_file_path("initscripts", "__startup__", ".py")
         )
-
-        # Ensure the copy of default python libraries
-        dependent_files: set[Path] = self.get_dependent_files(exe.base)
-        # Extra files like python3.dll need to be found
-        python_libs = self._default_bin_includes()
-        for bin_path in self.bin_path_includes:
-            for name in python_libs:
-                dependent_file = Path(bin_path, name)
-                if dependent_file.is_file():
-                    dependent_files.add(dependent_file)
-                    break
-
-        # Search the C runtimes, using the directory of the python libraries
-        # and the directories of the base executable
-        self._platform_add_extra_dependencies(dependent_files)
-
-        for source in dependent_files:
-            # Store dynamic libraries in appropriate location for platform.
-            self._copy_top_dependency(source)
-            # Once copied, it should be deleted from the list to ensure
-            # it will not be copied again.
-            name = os.path.normcase(source.name)
-            if name in self.bin_includes:
-                self.bin_includes.remove(name)
-                self.bin_excludes.append(name)
-
+        # copy the executable and its dependencies
         target_path = self.target_dir / exe.target_name
+        self._get_top_dependencies(exe.base)
         self._copy_file(
-            exe.base,
-            target_path,
-            copy_dependent_files=False,
-            include_mode=True,
+            exe.base, target_path, copy_dependent_files=True, include_mode=True
         )
+        if not os.access(target_path, os.W_OK):
+            mode = target_path.stat().st_mode
+            target_path.chmod(mode | stat.S_IWUSR)
 
         # copy a file with a the cx_freeze license into frozen application
         respath = get_resource_file_path(
@@ -327,13 +316,13 @@ class Freezer:
             include_mode=False,
         )
 
-        if not os.access(target_path, os.W_OK):
-            mode = target_path.stat().st_mode
-            target_path.chmod(mode | stat.S_IWUSR)
-
         # Add resources like version metadata and icon
         self._add_resources(exe)
 
+    @abstractmethod
+    def _get_top_dependencies(self, source: Path, target: Path) -> None:
+        """Called to get the dependencies of an executable."""
+
     def _platform_add_extra_dependencies(
         self, dependent_files: set[Path]
     ) -> None:
@@ -342,36 +331,31 @@ class Freezer:
         """
 
     @abstractmethod
-    def _copy_top_dependency(self, source: Path) -> None:
-        """Called for copying the top dependencies in _freeze_executable."""
-
     def _default_bin_excludes(self) -> list[str]:
         """Return the file names of libraries that need not be included because
         they would normally be expected to be found on the target system or
         because they are part of a package which requires independent
         installation anyway.
-        (overridden on Windows)
-        .
         """
-        return ["libclntsh.so", "libwtc9.so", "ldd"]
 
     def _default_bin_includes(self) -> list[str]:
         """Return the file names of libraries which must be included for the
         frozen executable to work.
-        (overriden on Windows)
-        .
+        (Overridden on Windows and macOS).
         """
-        python_shared_libs: list[str] = []
+        python_shared_libs: list[Path] = []
         name = sysconfig.get_config_var("INSTSONAME")
         if name:
             if name.endswith(".a"):
-                # Miniconda python Linux/macOS returns a static library.
-                if IS_MACOS:
-                    name = name.replace(".a", ".dylib")
-                else:
-                    name = name.replace(".a", ".so")
-            python_shared_libs.append(self._remove_version_numbers(name))
-        return python_shared_libs
+                # conda-forge returns a static library
+                name = name.replace(".a", ".so")
+            name = self._remove_version_numbers(name)
+            for bin_path in self._default_bin_path_includes():
+                fullname = Path(bin_path, name).resolve()
+                if fullname.is_file():
+                    python_shared_libs.append(fullname)
+                    break
+        return self._validate_bin_file(python_shared_libs)
 
     @abstractmethod
     def _default_bin_path_excludes(self) -> list[str]:
@@ -380,11 +364,16 @@ class Freezer:
         libraries.
         """
 
-    @abstractmethod
     def _default_bin_path_includes(self) -> list[str]:
         """Return the paths of directories which contain files that should
         be included.
+        (Overridden on Windows, inherited in macOS).
         """
+        bin_path = [
+            sysconfig.get_config_var("LIBDIR"),
+            sysconfig.get_config_var("DESTLIB"),
+        ]
+        return self._validate_bin_path(bin_path)
 
     def _get_module_finder(self) -> ModuleFinder:
         finder = ModuleFinder(
@@ -414,17 +403,6 @@ class Freezer:
             if not target.exists():
                 target.symlink_to(symlink, symlink_is_directory)
 
-    def _print_report(self, filename: Path, modules: list[Module]) -> None:
-        print(f"writing zip file {filename}\n")
-        print(f"  {'Name':<25} File")
-        print(f"  {'----':<25} ----")
-        for module in modules:
-            if module.path:
-                print("P", end="")
-            else:
-                print("m", end="")
-            print(f" {module.name:<25} {module.file or ''}")
-
     @staticmethod
     def _remove_version_numbers(filename: str) -> str:
         tweaked = False
@@ -438,22 +416,24 @@ class Freezer:
             return ".".join(parts)
         return filename
 
-    def _should_copy_file(self, path: Path) -> bool:
+    def _should_copy_file(self, path: Path) -> bool:  # noqa: PLR0911
         """Return true if the file should be copied to the target machine.
 
         This is done by checking the bin_includes and bin_excludes
         configuration variables using first the full file name, then just the
         base file name, then the file name without any version numbers.
         Then, bin_path_includes and bin_path_excludes are checked.
+        Finally, check the default variables.
 
         Files are included unless specifically excluded but inclusions take
         precedence over exclusions.
         """
+        dirname = path.parent
+        filename = Path(os.path.normcase(path.name))
+        filename_noversion = Path(self._remove_version_numbers(filename.name))
         # check the full path
         # check the file name by itself (with any included version numbers)
         # check the file name by itself (version numbers removed)
-        filename = Path(os.path.normcase(path.name))
-        filename_noversion = Path(self._remove_version_numbers(filename.name))
         for binfile in self.bin_includes:
             if (
                 path.match(binfile)
@@ -470,7 +450,6 @@ class Freezer:
                 return False
 
         # check the path for inclusion/exclusion
-        dirname = path.parent
         for binpath in self.bin_path_includes:
             try:
                 dirname.relative_to(binpath)
@@ -486,6 +465,28 @@ class Freezer:
             else:
                 return False
 
+        # check the default variables
+        for binfile in self.default_bin_includes:
+            if path.match(binfile):
+                return True
+        for binfile in self.default_bin_excludes:
+            if filename_noversion.match(binfile):
+                return False
+        for binpath in self.default_bin_path_includes:
+            try:
+                dirname.relative_to(binpath)
+            except ValueError:
+                pass
+            else:
+                return True
+        for binpath in self.default_bin_path_excludes:
+            try:
+                dirname.relative_to(binpath)
+            except ValueError:
+                pass
+            else:
+                return False
+
         return True
 
     def _validate_executables(
@@ -532,15 +533,11 @@ class Freezer:
         if bin_path is None:
             return []
         return [
-            os.fspath(path) for path in map(Path, bin_path) if path.is_dir()
+            os.fspath(path.resolve())
+            for path in map(Path, bin_path)
+            if path.is_dir()
         ]
 
-    def _verify_configuration(self) -> None:
-        self.bin_includes += self._default_bin_includes()
-        self.bin_excludes += self._default_bin_excludes()
-        self.bin_path_includes += self._default_bin_path_includes()
-        self.bin_path_excludes += self._default_bin_path_excludes()
-
     def _populate_zip_options(
         self,
         zip_include_packages: Sequence[str] | None,
@@ -591,6 +588,14 @@ class Freezer:
         cache_path = finder.cache_path
 
         modules = [m for m in finder.modules if m.name not in finder.excludes]
+        for module in finder.namespaces:
+            # if namespace package should be written to zip file, convert it
+            # to regular package, since then zipimport doesn't support PEP420
+            if module.in_file_system == 0:
+                module.code = compile(
+                    "", "__init__.py", "exec", dont_inherit=True
+                )
+                modules.append(module)
         modules.append(
             finder.include_file_as_module(
                 self.constants_module.create(cache_path, modules)
@@ -598,11 +603,6 @@ class Freezer:
         )
         modules.sort(key=lambda m: m.name)
 
-        if self.silent < 1:
-            self._print_report(filename, modules)
-        if self.silent < 2:
-            finder.report_missing_modules()
-
         target_lib_dir = filename.parent
         self._create_directory(target_lib_dir)
 
@@ -743,6 +743,9 @@ class Freezer:
             library_data = self.target_dir / "lib" / "library.dat"
             library_data.write_bytes(self.zip_filename.name.encode())
 
+        # to report
+        self.modules_copied = modules
+
     def freeze(self) -> None:
         """Do the freeze."""
         finder: ModuleFinder = self.finder
@@ -787,13 +790,51 @@ class Freezer:
         # do any platform-specific post-Freeze work
         self._post_freeze_hook()
 
+    def print_report(self) -> None:
+        """Display report:
+        - list of modules and packages;
+        - list of modules  that weren't found;
+        - list of dependencies that weren't found.
+        """
+        if self.silent < 1:
+            if self.zip_filename:
+                print(f"writing zip file {self.zip_filename}\n")
+            print(f"  {'Name':<25} File")
+            print(f"  {'----':<25} ----")
+            for module in self.modules_copied:
+                if module.path:
+                    print("P", end="")
+                else:
+                    print("m", end="")
+                print(f" {module.name:<25} {module.file or ''}")
+        if self.silent < 2:
+            self.finder.report_missing_modules()
+        if self.silent < 3:
+            # Display a list of dependencies that weren't found
+            names = {
+                name for name, value in self.linker_warnings.items() if value
+            }
+            copied = {file.name for file in self.files_copied}
+            missing = names.difference(copied)
+            if missing:
+                print("Missing dependencies:")
+                for name in sorted(missing):
+                    print(f"? {name}")
+                print("This is not necessarily a problem - the ", end="")
+                print("dependencies may not be needed on this platform.\n")
+
 
 class WinFreezer(Freezer, PEParser):
     """Freezer base class for Windows OS."""
 
     def __init__(self, *args, **kwargs) -> None:
         Freezer.__init__(self, *args, **kwargs)
-        PEParser.__init__(self, self.path, self.bin_path_includes, self.silent)
+        PEParser.__init__(
+            self,
+            self.path,
+            self.default_bin_path_includes + self.bin_path_includes,
+            self.silent,
+        )
 
     def _add_resources(self, exe: Executable) -> None:
         target_path: Path = self.target_dir / exe.target_name
@@ -890,31 +931,6 @@ class WinFreezer(Freezer, PEParser):
             if self.silent < 3:
                 print("WARNING:", exc)
 
-    def _copy_top_dependency(self, source: Path) -> None:
-        """Called for copying certain top dependencies in _freeze_executable.
-        We need this as a separate method so that it can be overridden on
-        Darwin and Windows.
-        """
-        # top dependencies go into build root directory on windows
-        # MS VC runtimes are handled in _copy_file/_pre_copy_hook
-        # msys2 libpython depends on libgcc_s_seh and libwinpthread dlls
-        # conda-forge python3x.dll depends on zlib.dll
-        target_dir = self.target_dir
-        target = target_dir / source.name
-        self._copy_file(
-            source, target, copy_dependent_files=False, include_mode=True
-        )
-        for dependent_source in self.get_dependent_files(source):
-            if IS_MINGW:
-                if self._should_copy_file(dependent_source):
-                    self._copy_top_dependency(dependent_source)
-            else:
-                self._copy_file(
-                    source=dependent_source,
-                    target=target_dir / dependent_source.name,
-                    copy_dependent_files=True,
-                )
-
     def _pre_copy_hook(self, source: Path, target: Path) -> tuple[Path, Path]:
         """Prepare the source and target paths. Also, adjust the target of
         C runtime libraries.
@@ -922,7 +938,7 @@ class WinFreezer(Freezer, PEParser):
         # fix the target path for C runtime files
         if any(filter(target.match, self.runtime_files)):
             target = self.target_dir / target.name
-        return source, target
+        return super()._pre_copy_hook(source, target)
 
     def _post_copy_hook(
         self,
@@ -930,26 +946,32 @@ class WinFreezer(Freezer, PEParser):
         target: Path,
         copy_dependent_files: bool,
     ) -> None:
-        if (
+        if not (
             copy_dependent_files
             and source not in self.finder.excluded_dependent_files
         ):
-            library_dir = self.target_dir / "lib"
-            source_dir = source.parent
-            target_dir = target.parent
-            platform_bin_path = self._platform_bin_path
-            for dependent_source in self.get_dependent_files(source):
-                if not self._should_copy_file(dependent_source):
-                    continue
-                # put the dependency in the target_dir (except C runtime)
-                dependent_srcdir = dependent_source.parent
-                dependent_name = dependent_source.name
+            return
+        library_dir = self.target_dir / "lib"
+        source_dir = source.parent
+        target_dir = target.parent
+        lib_files = self.finder.lib_files
+        platform_bin_path = self._platform_bin_path
+        for dependent in self.get_dependent_files(source):
+            if not self._should_copy_file(dependent):
+                continue
+            dependent_source = dependent.resolve()
+            dependent_name = dependent_source.name
+            lib_file = lib_files.get(dependent_source)
+            if lib_file:
+                dependent_target = self.target_dir / lib_file
+            else:
+                # put the dependency (relatively) in the target_dir subtree
+                # this is possible with most packages installed by pip
                 try:
-                    # dependency located with source or in a subdirectory
                     relative = dependent_source.relative_to(source_dir)
-                    dependent_target = target_dir / relative
                 except ValueError:
                     # check if dependency is on default binaries path
+                    dependent_srcdir = dependent_source.parent
                     if dependent_srcdir in platform_bin_path:
                         dependent_target = library_dir / dependent_name
                     else:
@@ -961,13 +983,16 @@ class WinFreezer(Freezer, PEParser):
                             dependent_target = Path(*parts) / dependent_name
                         except ValueError:
                             dependent_target = target_dir / dependent_name
+                else:
+                    # dependency located with source or in a subdirectory
+                    dependent_target = target_dir / relative
                 # make sure the dependency is in the correct directory because
                 # it cannot be outside the library_dir directory subtree
                 try:
                     dependent_target.relative_to(library_dir)
                 except ValueError:
                     dependent_target = library_dir / dependent_name
-                else:
+                if 0:  # else:
                     _, dependent_target = self._pre_copy_hook(
                         dependent_source, dependent_target
                     )
@@ -976,25 +1001,31 @@ class WinFreezer(Freezer, PEParser):
                             if file.match(dependent_name):
                                 dependent_target = file
                                 break
-                self._copy_file(
-                    dependent_source, dependent_target, copy_dependent_files
-                )
+            self._copy_file(
+                dependent_source, dependent_target, copy_dependent_files
+            )
 
     def _default_bin_excludes(self) -> list[str]:
         return ["comctl32.dll", "oci.dll"]
 
     def _default_bin_includes(self) -> list[str]:
-        python_shared_libs: list[str] = []
         name = sysconfig.get_config_var("INSTSONAME")
         if name:
             # MSYS2 python returns a static library.
-            python_shared_libs.append(name.replace(".dll.a", ".dll"))
+            names = [name.replace(".dll.a", ".dll")]
         else:
-            python_shared_libs += [
+            names = [
                 f"python{sys.version_info[0]}.dll",
                 f"python{sys.version_info[0]}{sys.version_info[1]}.dll",
             ]
-        return python_shared_libs
+        python_shared_libs: list[Path] = []
+        for name in names:
+            for bin_path in self._default_bin_path_includes():
+                fullname = Path(bin_path, name).resolve()
+                if fullname.is_file():
+                    python_shared_libs.append(fullname)
+                    break
+        return self._validate_bin_file(python_shared_libs)
 
     def _default_bin_path_excludes(self) -> list[str]:
         system_dir = GetSystemDir()
@@ -1004,6 +1035,17 @@ class WinFreezer(Freezer, PEParser):
     def _default_bin_path_includes(self) -> list[str]:
         return self._validate_bin_path(sys.path + self._platform_bin_path)
 
+    def _get_top_dependencies(self, source: Path) -> None:
+        # executable dependencies go into build root directory on windows
+        # msys2 libpython depends on libgcc_s_seh and libwinpthread dlls
+        # conda-forge python3x.dll depends on zlib.dll
+        finder = self.finder
+        for filename in self.get_dependent_files(source):
+            path = filename.resolve()
+            if path not in finder.lib_files and self._should_copy_file(path):
+                finder.lib_files.setdefault(path, path.name)
+                self._get_top_dependencies(path)
+
     @cached_property
     def _platform_bin_path(self) -> list[Path]:
         # try to find the paths (windows, conda-forge, msys2/mingw)
@@ -1028,7 +1070,7 @@ class WinFreezer(Freezer, PEParser):
             if dest_relative:
                 paths.add(prefix / dest_relative)
         # return only valid paths
-        return [path for path in paths if path.is_dir()]
+        return [path.resolve() for path in paths if path.is_dir()]
 
     def _platform_add_extra_dependencies(
         self, dependent_files: set[Path]
@@ -1059,9 +1101,75 @@ class DarwinFreezer(Freezer, Parser):
 
     def __init__(self, *args, **kwargs) -> None:
         Freezer.__init__(self, *args, **kwargs)
-        Parser.__init__(self, self.path, self.bin_path_includes, self.silent)
+        Parser.__init__(
+            self,
+            self.path,
+            self.default_bin_path_includes + self.bin_path_includes,
+            self.silent,
+        )
         self.darwin_tracker: DarwinFileTracker = DarwinFileTracker()
 
+    def _default_bin_excludes(self) -> list[str]:
+        return []
+
+    def _default_bin_includes(self) -> list[str]:
+        python_shared_libs: list[Path] = []
+        # Check for distributed "cx_Freeze/bases/lib/Python"
+        name = "Python"
+        for bin_path in self._default_bin_path_includes():
+            fullname = Path(bin_path, name).resolve()
+            if fullname.is_file():
+                python_shared_libs.append(fullname)
+                break
+        if not python_shared_libs:
+            # INSTSONAME returns a relative path like
+            # "Python.framework/Versions/3.12/Python" if PYTHONFRAMEWORK is
+            # set, or can return a .dylib that does not exist.
+            # On the other hand, in conda-forge it returns a static library .a
+            # instead of a .dylib.
+            name = sysconfig.get_config_var("INSTSONAME")
+            if name.endswith(".a"):
+                name = name.replace(".a", ".dylib")  # fix for conda-forge
+            for bin_path in self._default_bin_path_includes():
+                fullname = Path(bin_path, name).resolve()
+                if fullname.is_file():
+                    python_shared_libs.append(fullname)
+                    break
+        return self._validate_bin_file(python_shared_libs)
+
+    def _default_bin_path_excludes(self) -> list[str]:
+        return ["/lib", "/usr/lib", "/System/Library/Frameworks"]
+
+    def _default_bin_path_includes(self) -> list[str]:
+        # use macpython distributed files if available
+        bases_lib = get_resource_file_path("bases", "lib", "")
+        if bases_lib:
+            return self._validate_bin_path([bases_lib])
+        # use default
+        return super()._default_bin_path_includes()
+
+    def _get_top_dependencies(self, source: Path) -> None:
+        # this recovers the cached MachOReference pointers to the files
+        # found by the get_dependent_files calls made previously (if any).
+        # If one is found, pass into _copy_file.
+        # We need to do this so the file knows what file referenced it,
+        # and can therefore calculate the appropriate rpath.
+        # (We only cache one reference.)
+        # =cached_reference = self.darwin_tracker.getCachedReferenceTo(source)
+        # =darwin_file = DarwinFile(source, cached_reference)
+        dependent_files = self.get_dependent_files(source)  # darwin_file
+        dependent_files.update(set(map(Path, self.default_bin_includes)))
+        for dep_source in dependent_files:
+            dep_target = self.target_dir / "lib" / dep_source.name
+            reference = self.darwin_tracker.getCachedReferenceTo(dep_source)
+            self._copy_file_recursion(
+                dep_source,
+                dep_target,
+                copy_dependent_files=True,
+                include_mode=True,
+                reference=reference,
+            )
+
     def _post_freeze_hook(self) -> None:
         self.darwin_tracker.finalizeReferences()
         # Make all references to libraries relative
@@ -1152,38 +1260,6 @@ class DarwinFreezer(Freezer, Parser):
             reference=reference,
         )
 
-    def _copy_top_dependency(self, source: Path) -> None:
-        """Called for copying certain top dependencies. We need this as a
-        separate function so that it can be overridden on Darwin
-        (to interact with the DarwinTools system).
-        """
-        target = self.target_dir / "lib" / source.name
-
-        # this recovers the cached MachOReference pointers to the files
-        # found by the get_dependent_files calls made previously (if any).
-        # If one is found, pass into _copy_file.
-        # We need to do this so the file knows what file referenced it,
-        # and can therefore calculate the appropriate rpath.
-        # (We only cache one reference.)
-        cached_reference = self.darwin_tracker.getCachedReferenceTo(source)
-        self._copy_file_recursion(
-            source,
-            target,
-            copy_dependent_files=True,
-            include_mode=True,
-            reference=cached_reference,
-        )
-
-    def _default_bin_path_excludes(self) -> list[str]:
-        return ["/lib", "/usr/lib", "/System/Library/Frameworks"]
-
-    def _default_bin_path_includes(self) -> list[str]:
-        bin_path = [
-            sysconfig.get_config_var("LIBDIR"),
-            sysconfig.get_config_var("DESTLIB"),
-        ]
-        return self._validate_bin_path(bin_path)
-
     def get_dependent_files(
         self, filename: Path, darwinFile: DarwinFile | None = None
     ) -> set[Path]:
@@ -1221,31 +1297,79 @@ class LinuxFreezer(Freezer, ELFParser):
     def __init__(self, *args, **kwargs) -> None:
         Freezer.__init__(self, *args, **kwargs)
         ELFParser.__init__(
-            self, self.path, self.bin_path_includes, self.silent
+            self,
+            self.path,
+            self.default_bin_path_includes + self.bin_path_includes,
+            self.silent,
         )
 
+    def _default_bin_excludes(self) -> list[str]:
+        # https://github.com/AppImageCommunity/pkg2appimage/blob/master/excludelist
+        return [
+            "ldd",  # old alpine uses ldd instead of ld.so
+            "ld.so",
+            "ld64.so",
+            "ld-linux.so",
+            "ld-linux-*.so",  # ld-linux-x86-64.so.2, ld-linux-aarch64.so.2
+            "libc.so",
+            "libdl.so",
+            "libm.so",
+            "libpthread.so",
+            "libutil.so",
+            # musllinux
+            "ld-musl-*.so",
+        ]
+
+    def _default_bin_path_excludes(self) -> list[str]:
+        return [
+            "/lib",
+            "/lib32",
+            "/lib64",
+            "/usr/lib",
+            "/usr/lib32",
+            "/usr/lib64",
+        ]
+
+    def _get_top_dependencies(self, source: Path) -> None:  # noqa: ARG002
+        finder = self.finder
+        for path in map(Path, self.default_bin_includes):
+            finder.lib_files.setdefault(path, f"lib/{path.name}")
+
     def _post_copy_hook(
         self,
         source: Path,
         target: Path,
         copy_dependent_files: bool,
     ) -> None:
-        if (
+        if not (
             copy_dependent_files
             and source not in self.finder.excluded_dependent_files
         ):
-            source_dir = source.parent
-            target_dir = target.parent
-            fix_rpath = set()
-            for dependent in self.get_dependent_files(source):
-                if not self._should_copy_file(dependent):
-                    continue
-                dependent_file = dependent.resolve()
+            return
+        source_dir = source.parent
+        target_dir = target.parent
+        lib_files = self.finder.lib_files
+        fix_rpath = set()
+        fix_needed = {}
+        for dependent in self.get_dependent_files(source):
+            if not self._should_copy_file(dependent):
+                continue
+            dependent_source = dependent.resolve()
+            dependent_name = dependent_source.name
+            lib_file = lib_files.get(dependent_source)
+            if lib_file:
+                dependent_target = self.target_dir / lib_file
+                try:
+                    relative = dependent_target.relative_to(target_dir)
+                except ValueError:
+                    relative = Path(
+                        os.path.relpath(dependent_target, target_dir)
+                    )
+            else:
                 # put the dependency (relatively) in the target_dir subtree
                 # this is possible with most packages installed by pip
-                dependent_name = dependent_file.name
                 try:
-                    relative = dependent_file.relative_to(source_dir)
+                    relative = dependent_source.relative_to(source_dir)
                 except ValueError:
                     # put the dependency in target_dir along with the binary
                     # file being copied, unless the dependency has already been
@@ -1264,40 +1388,15 @@ class LinuxFreezer(Freezer, ELFParser):
                                 dependent_target = file
                             break
                 else:
+                    # dependency located with source or in a subdirectory
                     dependent_target = target_dir / relative
-                dep_libs = os.fspath(relative.parent)
-                fix_rpath.add(dep_libs)
-                self._copy_file(
-                    dependent_file, dependent_target, copy_dependent_files
-                )
-                if dependent.name != dependent_name:
-                    self.replace_needed(target, dependent.name, dependent_name)
-            if fix_rpath:
-                has_rpath = self.get_rpath(target)
-                rpath = ":".join(f"$ORIGIN/{r}" for r in fix_rpath)
-                if has_rpath != rpath:
-                    self.set_rpath(target, rpath)
-
-    def _copy_top_dependency(self, source: Path) -> None:
-        """Called for copying the top dependencies in _freeze_executable."""
-        target = self.target_dir / "lib" / source.name
-        self._copy_file(
-            source, target, copy_dependent_files=True, include_mode=True
-        )
-
-    def _default_bin_path_excludes(self) -> list[str]:
-        return [
-            "/lib",
-            "/lib32",
-            "/lib64",
-            "/usr/lib",
-            "/usr/lib32",
-            "/usr/lib64",
-        ]
-
-    def _default_bin_path_includes(self) -> list[str]:
-        bin_path = [
-            sysconfig.get_config_var("LIBDIR"),
-            sysconfig.get_config_var("DESTLIB"),
-        ]
-        return self._validate_bin_path(bin_path)
+            fix_rpath.add(f"$ORIGIN/{relative.parent.as_posix()}")
+            self._copy_file(
+                dependent_source, dependent_target, copy_dependent_files
+            )
+            if dependent.name != dependent_name:
+                fix_needed.setdefault(dependent.name, dependent_name)
+        if fix_rpath:
+            self.set_rpath(target, ":".join(fix_rpath))
+        for needed_old, needed_new in fix_needed.items():
+            self.replace_needed(target, needed_old, needed_new)
