@@ -1,10 +1,9 @@
 """A collection of functions which are triggered automatically by finder when
-scipy package is included.
+scikit-learn package is included.
 """
 
 from __future__ import annotations
 
-import os
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
@@ -25,4 +24,10 @@ def load_sklearn__distributor_init(
         code_string = code_string.replace(
             "libs_path =", "libs_path = __import__('sys').frozen_dir  #"
         )
-        module.code = compile(code_string, os.fspath(module.file), "exec")
+        module.code = compile(
+            code_string,
+            module.file.as_posix(),
+            "exec",
+            dont_inherit=True,
+            optimize=finder.optimize,
+        )
