@@ -3,7 +3,7 @@ import sys
 from tkinter import Button, Label, PhotoImage, TclVersion, Tk
 
 
-def _test():
+def _test() -> None:
     root = Tk()
     text = (
         f"This is Tcl/Tk version {TclVersion}\n"
@@ -11,11 +11,11 @@ def _test():
     )
     label = Label(root, text=text)
     label.pack()
-    test = Button(
+    test_button = Button(
         root,
         text="Click me!",
         command=lambda root=root: root.test.configure(
-            text="[%s]" % root.test["text"]
+            text=f"[{root.test['text']}]"
         ),
     )
     # icon located in current directory is copied to "share" folder when frozen
@@ -26,11 +26,11 @@ def _test():
         else os.path.join(os.path.dirname(__file__), "logox128.png")
     )
     icon = PhotoImage(file=icon_file)
-    root.tk.call("wm", "iconphoto", root._w, icon)
-    test.pack()
-    root.test = test
-    quit = Button(root, text="QUIT", command=root.destroy)
-    quit.pack()
+    root.tk.call("wm", "iconphoto", root._w, icon)  # noqa: SLF001
+    test_button.pack()
+    root.test = test_button
+    quit_button = Button(root, text="QUIT", command=root.destroy)
+    quit_button.pack()
     # The following three commands are needed so the window pops
     # up on top on Windows...
     root.iconify()
